/*
 * Based on gps puzzle box from: https://randomnerdtutorials.com
 *
 * Based on the example TinyGPS++ from arduiniana.org
 */
 
#include <TinyGPSPlus.h>
#include <SoftwareSerial.h>

static const int RXPin = 4, TXPin = 3;
static const uint32_t GPSBaud = 9600;

//OLED stuff
  #include "ssd1306.h"
  #include "nano_gfx.h"

// The TinyGPSPlus object
TinyGPSPlus gps;

// The serial connection to the GPS device
SoftwareSerial ss(RXPin, TXPin);

//how far, and what angle to go at
int DISTANCE = 0;
int DEGREES = 0;

//refresh screen rate
unsigned long LAST_REFRESH = 1000;
long REFRESH_RATE = 2000;

void setup(){
  Serial.begin(115200);   //I like having a serial connection to a computer so I can view more information
  ss.begin(GPSBaud);
  ssd1306_128x64_i2c_init();
  Serial.println("Hello");
}

void loop(){
  // This sketch displays information every time a new sentence is correctly encoded.
  while (ss.available() > 0){
    gps.encode(ss.read());
    if (gps.location.isUpdated()){

      if (gps.location.isValid()) {

      //Discovery Park: 47.65496388084886, -122.4106509320196
      //distance to Discovery Park spot
      static const double PARK_LAT = 47.654963, PARK_LON = -122.410650;
      double distanceToPark =
        TinyGPSPlus::distanceBetween(
            gps.location.lat(),
            gps.location.lng(),
            PARK_LAT, 
            PARK_LON);
      double courseToPark =
        TinyGPSPlus::courseTo(
            gps.location.lat(),
            gps.location.lng(),
            PARK_LAT, 
            PARK_LON);
            
      DISTANCE = int(distanceToPark);   //display the distance on screen as this distance in meters
      DEGREES = int(courseToPark);


      if((millis() - LAST_REFRESH) > REFRESH_RATE){
        //update serial information if we are sending stuff to the serial port
        //by comparing the difference between the millis() times, we avoid overflow errors (which only happen every 50 days)
        // Latitude in degrees (double)
        Serial.print("Latitude= "); 
        Serial.print(gps.location.lat(), 6);      
        // Longitude in degrees (double)
        Serial.print(" Longitude= "); 
        Serial.println(gps.location.lng(), 6); 

        Serial.print(F("Park     Distance: "));
        Serial.print(distanceToPark/1000, 6);
        Serial.print(F(" km Course-to="));
        Serial.print(courseToPark, 6);
        Serial.print(F(" degrees ["));
        Serial.print(TinyGPSPlus::cardinal(courseToPark));
        Serial.println(F("]"));

        Serial.print("Distance to travel: ");
        Serial.println(DISTANCE);
        

        
        //update screen if it's been a second
        ssd1306_clearScreen( );
        ssd1306_normalMode();
        ssd1306_setFixedFont(ssd1306xled_font8x16); //set font
        delay(5);    //give the screen some time to catch up
        String Distance_STRING = String(int(DISTANCE)) + " m"; // + String(distanceToPark/1000, 2)
        ssd1306_printFixed(0,  0, Distance_STRING.c_str(), STYLE_BOLD);
        String Heading_STRING = String(int(DEGREES)) + " " + String(TinyGPSPlus::cardinal(courseToPark));
        ssd1306_printFixed(0,  16, Heading_STRING.c_str(), STYLE_BOLD);
        //now, print the current coordinates from the GPS unit, so you can use them to locate yourself when walking around
        /*
        String Lat_STRING = String(long((gps.location.lat())*1000000));
        ssd1306_printFixed(0,  32, Lat_STRING.c_str(), STYLE_BOLD);
        String Long_STRING = String(long((gps.location.lng())*1000000));
        ssd1306_printFixed(0,  48, Long_STRING.c_str(), STYLE_BOLD);
        LAST_REFRESH = millis();   //reset refresh time
        */
  
        Serial.println("Information on screen");
        Serial.println(Distance_STRING);
        Serial.println(Heading_STRING);
        // Serial.println(Lat_STRING);  //these are multiplied by 1,000,000
        // Serial.println(Long_STRING); //these are multiplied by 1,000,000
      }
      
    }
      
    
    }
  }
  if(((millis() - LAST_REFRESH) > REFRESH_RATE) && (gps.satellites.value() == 0)){
    // if we don't see any satellites, and it's time to refresh, inform everyone
    // Number of satellites in use (u32)
    Serial.print("Number of satellites in use = "); 
    Serial.println(gps.satellites.value()); 
    Serial.println("No signal");
    //display no signal on screen
    ssd1306_clearScreen( );
    ssd1306_normalMode();
    delay(5);    //give the screen some time to catch up
    String Signal_STRING = "No Signal"; // + String(distanceToBFDay/1000, 2)
    ssd1306_setFixedFont(ssd1306xled_font8x16); //set font
    ssd1306_printFixed(0,  0, Signal_STRING.c_str(), STYLE_BOLD);
    LAST_REFRESH = millis();   //reset time to now to refresh in another second
    
  }
  
}
